@@ -5,10 +5,11 @@ use warnings;
 
 BEGIN {
 	$Reply::Plugin::TypeTiny::AUTHORITY = 'cpan:TOBYINK';
-	$Reply::Plugin::TypeTiny::VERSION   = '0.022';
+	$Reply::Plugin::TypeTiny::VERSION   = '1.000005';
 };
 
-use base 'Reply::Plugin';
+require Reply::Plugin;
+our @ISA = 'Reply::Plugin';
 
 use Scalar::Util qw(blessed);
 use Term::ANSIColor;
@@ -17,13 +18,13 @@ sub mangle_error {
 	my $self  = shift;
 	my ($err) = @_;
 	
-	if (blessed $err and $err->isa("Type::Exception::Assertion"))
+	if (blessed $err and $err->isa("Error::TypeTiny::Assertion"))
 	{
 		my $explain = $err->explain;
 		if ($explain)
 		{
 			print color("cyan");
-			print "Type::Exception::Assertion explain:\n";
+			print "Error::TypeTiny::Assertion explain:\n";
 			$self->_explanation($explain, "");
 			local $| = 1;
 			print "\n";
@@ -65,6 +66,11 @@ __END__
 
 Reply::Plugin::TypeTiny - improved type constraint exceptions in Reply
 
+=head1 STATUS
+
+This module is not covered by the
+L<Type-Tiny stability policy|Type::Tiny::Manual::Policies/"STABILITY">.
+
 =head1 DESCRIPTION
 
 This is a small plugin to improve error messages in L<Reply>.
@@ -83,7 +89,7 @@ L<http://rt.cpan.org/Dist/Display.html?Queue=Type-Tiny>.
 
 =head1 SEE ALSO
 
-L<Type::Exception::Assertion>, L<Reply>.
+L<Error::TypeTiny::Assertion>, L<Reply>.
 
 =head1 AUTHOR
 
@@ -91,7 +97,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2013 by Toby Inkster.
+This software is copyright (c) 2013-2014 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
