@@ -6,17 +6,19 @@ use warnings;
 
 BEGIN {
 	$Type::Registry::AUTHORITY = 'cpan:TOBYINK';
-	$Type::Registry::VERSION   = '0.022';
+	$Type::Registry::VERSION   = '1.000005';
 }
 
-use Exporter::TypeTiny qw( mkopt _croak );
+use Exporter::Tiny qw( mkopt );
 use Scalar::Util qw( refaddr );
 use Type::Parser qw( eval_type );
-use Types::TypeTiny qw( ArrayLike );
+use Types::TypeTiny qw( CodeLike ArrayLike to_TypeTiny );
 
-use base "Exporter::TypeTiny";
+our @ISA = 'Exporter::Tiny';
 our @EXPORT_OK = qw(t);
 
+sub _croak ($;@) { require Error::TypeTiny; goto \&Error::TypeTiny::croak }
+
 sub _exporter_expand_sub
 {
 	my $class = shift;
@@ -66,34 +68,95 @@ sub add_types
 	for my $opt (@$opts)
 	{
 		my ($lib, $types) = @_;
-		$types ||= [qw/-types/];
 		
 		$lib =~ s/^-/Types::/;
 		eval "require $lib";
-		$lib->isa("Type::Library") || $lib eq 'Types::TypeTiny'
-			or _croak("%s is not a type library", $lib);
-		
-		ArrayLike->check($types)
-			or _croak("Expected arrayref following '%s'; got %s", $lib, $types);
 		
 		my %hash;
-		$lib->import({into => \%hash}, @$types);
+		
+		if ($lib->isa("Type::Library") or $lib eq 'Types::TypeTiny')
+		{
+			$types ||= [qw/-types/];
+			ArrayLike->check($types)
+				or _croak("Expected arrayref following '%s'; got %s", $lib, $types);
+			
+			$lib->import({into => \%hash}, @$types);
+			$hash{$_} = &{$hash{$_}}() for keys %hash;
+		}
+		elsif ($lib->isa("MooseX::Types::Base"))
+		{
+			$types ||= [];
+			ArrayLike->check($types) && (@$types == 0)
+				or _croak("Library '%s' is a MooseX::Types type constraint library. No import options currently supported", $lib);
+			
+			require Moose::Util::TypeConstraints;
+			my $moosextypes = $lib->type_storage;
+			for my $name (sort keys %$moosextypes)
+			{
+				my $tt = to_TypeTiny(
+					Moose::Util::TypeConstraints::find_type_constraint($moosextypes->{$name})
+				);
+				$hash{$name} = $tt;
+			}
+		}
+		elsif ($lib->isa("MouseX::Types::Base"))
+		{
+			$types ||= [];
+			ArrayLike->check($types) && (@$types == 0)
+				or _croak("Library '%s' is a MouseX::Types type constraint library. No import options currently supported", $lib);
+			
+			require Mouse::Util::TypeConstraints;
+			my $moosextypes = $lib->type_storage;
+			for my $name (sort keys %$moosextypes)
+			{
+				my $tt = to_TypeTiny(
+					Mouse::Util::TypeConstraints::find_type_constraint($moosextypes->{$name})
+				);
+				$hash{$name} = $tt;
+			}
+		}
+		else
+		{
+			_croak("%s is not a type library", $lib);
+		}
 		
 		for my $key (sort keys %hash)
 		{
 			exists($self->{$key})
+				and $self->{$key}{uniq} != $hash{$key}{uniq}
 				and _croak("Duplicate type name: %s", $key);
-			$self->{$key} = $hash{$key}->();
+			$self->{$key} = $hash{$key};
 		}
 	}
 	$self;
 }
 
+sub add_type
+{
+	my $self = shift;
+	my ($type, $name) = @_;
+	$type = to_TypeTiny($type);
+	$name ||= do {
+		$type->is_anon
+			and _croak("Expected named type constraint; got anonymous type constraint");
+		$type->name;
+	};
+	
+	exists($self->{$name})
+		and $self->{$name}{uniq} != $type->{uniq}
+		and _croak("Duplicate type name: %s", $name);
+	
+	$self->{$name} = $type;
+	$self;
+}
+
 sub alias_type
 {
 	my $self = shift;
 	my ($old, @new) = @_;
-	$self->{$_} = $self->{$old} for @new;
+	my $lookup = eval { $self->lookup($old) }
+		or _croak("Expected existing type constraint name; got '$old'");
+	$self->{$_} = $lookup for @new;
 	$self;
 }
 
@@ -112,6 +175,44 @@ sub simple_lookup
 	return;
 }
 
+sub foreign_lookup
+{
+	my $self = shift;
+	
+	return $_[1] ? () : $self->simple_lookup($_[0], 1)
+		unless $_[0] =~ /^(.+)::(\w+)$/;
+	
+	my $library  = $1;
+	my $typename = $2;
+	
+	eval "require $library;";
+	
+	if ( $library->isa('MooseX::Types::Base') )
+	{
+		require Moose::Util::TypeConstraints;
+		my $type = Moose::Util::TypeConstraints::find_type_constraint(
+			$library->get_type($typename)
+		) or return;
+		return to_TypeTiny($type);
+	}
+	
+	if ( $library->isa('MouseX::Types::Base') )
+	{
+		require Mouse::Util::TypeConstraints;
+		my $sub  = $library->can($typename) or return;
+		my $type = Mouse::Util::TypeConstraints::find_type_constraint($sub->()) or return;
+		return to_TypeTiny($type);
+	}
+	
+	if ( $library->can("get_type") )
+	{
+		my $type = $library->get_type($typename);
+		return to_TypeTiny($type);
+	}
+	
+	return;
+}
+
 sub lookup
 {
 	my $self = shift;
@@ -119,6 +220,42 @@ sub lookup
 	$self->simple_lookup(@_) or eval_type($_[0], $self);
 }
 
+sub make_union
+{
+	my $self = shift;
+	my (@types) = @_;
+	
+	require Type::Tiny::Union;
+	return "Type::Tiny::Union"->new(type_constraints => \@types);
+}
+
+sub make_intersection
+{
+	my $self = shift;
+	my (@types) = @_;
+	
+	require Type::Tiny::Intersection;
+	return "Type::Tiny::Intersection"->new(type_constraints => \@types);
+}
+
+sub make_class_type
+{
+	my $self = shift;
+	my ($class) = @_;
+	
+	require Type::Tiny::Class;
+	return "Type::Tiny::Class"->new(class => $class);
+}
+
+sub make_role_type
+{
+	my $self = shift;
+	my ($role) = @_;
+	
+	require Type::Tiny::Role;
+	return "Type::Tiny::Role"->new(role => $role);
+}
+
 sub AUTOLOAD
 {
 	my $self = shift;
@@ -134,6 +271,20 @@ sub DESTROY
 	return;
 }
 
+DELAYED: {
+	our %DELAYED;
+	for my $package (sort keys %DELAYED)
+	{
+		my $reg   = __PACKAGE__->for_class($package);
+		my $types = $DELAYED{$package};
+		
+		for my $name (sort keys %$types)
+		{
+			$reg->add_type($types->{$name}, $name);
+		}
+	}
+}
+
 1;
 
 __END__
@@ -150,6 +301,8 @@ Type::Registry - a glorified hashref for looking up type constraints
 
 =head1 SYNOPSIS
 
+=for test_synopsis no warnings qw(misc);
+
    package Foo::Bar;
    
    use Type::Registry;
@@ -198,26 +351,8 @@ Alternatively:
 
 =head1 STATUS
 
-Type::Registry (and L<Type::Parser>) is currently a pretty isolated part
-of this distribution. It seems like something that would be useful, but
-it's not heavily integrated with everything else. In particular, if you
-do:
-
-   use Type::Registry qw(t);
-   use Types::Standard -types;
-
-Then the C<Str>, C<Num>, etc keywords imported from L<Types::Standard> will
-work fine, but C<< t->lookup("Str") >> and C<< t->lookup("Num") >> will fail,
-because importing types from a library does not automatically add them to
-your registry.
-
-Clearly some kind of integration is desirable between Type::Registry and
-L<Type::Library>, but exactly what form that will take is still to be
-decided.
-
-So if you decide to use Type::Registry, be aware of its somewhat experimental
-status. It's not likely to disappear completely, but there may be changes
-ahead.
+This module is covered by the
+L<Type-Tiny stability policy|Type::Tiny::Manual::Policies/"STABILITY">.
 
 =head1 DESCRIPTION
 
@@ -237,6 +372,9 @@ Create a new glorified hashref.
 Create or return the existing glorified hashref associated with the given
 class.
 
+Note that any type constraint you have imported from Type::Library-based
+type libraries will be automatically available in your class' registry.
+
 =item C<< for_me >>
 
 Create or return the existing glorified hashref associated with the caller.
@@ -265,6 +403,24 @@ Otherwise, imports all types from the library.
       -Standard => ['HashRef' => { -as => 'RealHash' }],
    );
 
+L<MooseX::Types> (and experimentally, L<MouseX::Types>) libraries can
+also be added this way, but I<< cannot be followed by an arrayref of
+types to import >>.
+
+=item C<< add_type($type, $name) >>
+
+The long-awaited singular form of C<add_types>. Given a type constraint
+object, adds it to the registry with a given name. The name may be
+omitted, in which case C<< $type->name >> is called, and Type::Registry
+will throw an error if C<< $type >> is anonymous. If a name is explicitly
+given, Type::Registry cares not one wit whether the type constraint is
+anonymous.
+
+This method can even add L<MooseX::Types> and L<MouseX::Types> type
+constraints; indeed anything that can be handled by L<Types::TypeTiny>'s
+C<to_TypeTiny> function. (Bear in mind that to_TypeTiny I<always> results
+in an anonymous type constraint, so C<< $name >> will be required.)
+
 =item C<< alias_type($oldname, $newname) >>
 
 Create an alias for an existing type.
@@ -275,6 +431,11 @@ Look up a type in the registry by name.
 
 Returns undef if not found.
 
+=item C<< foreign_lookup($name) >>
+
+Like C<simple_lookup>, but if the type name contains "::", will attempt
+to load it from a type library. (And will attempt to load that module.)
+
 =item C<< lookup($name) >>
 
 Look up by name, with a DSL.
@@ -294,6 +455,13 @@ The DSL can be summed up as:
 
 Croaks if not found.
 
+=item C<< make_union(@constraints) >>,
+C<< make_intersection(@constraints) >>,
+C<< make_class_type($class) >>,
+C<< make_role_type($role) >>
+
+Convenience methods for creating certain common type constraints.
+
 =item C<< AUTOLOAD >>
 
 Overloaded to call C<lookup>.
@@ -328,7 +496,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2013 by Toby Inkster.
+This software is copyright (c) 2013-2014 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
