@@ -10,25 +10,38 @@ BEGIN {
 
 BEGIN {
 	$Type::Tiny::Class::AUTHORITY = 'cpan:TOBYINK';
-	$Type::Tiny::Class::VERSION   = '0.022';
+	$Type::Tiny::Class::VERSION   = '1.000005';
 }
 
 use Scalar::Util qw< blessed >;
 
-sub _croak ($;@) { require Type::Exception; goto \&Type::Exception::croak }
+sub _croak ($;@) { require Error::TypeTiny; goto \&Error::TypeTiny::croak }
 
-use base "Type::Tiny";
+use Type::Tiny ();
+our @ISA = 'Type::Tiny';
 
 sub new {
 	my $proto = shift;
 	return $proto->class->new(@_) if blessed $proto; # DWIM
 	
-	my %opts = @_;
+	my %opts = (@_==1) ? %{$_[0]} : @_;
 	_croak "Class type constraints cannot have a parent constraint passed to the constructor" if exists $opts{parent};
 	_croak "Class type constraints cannot have a constraint coderef passed to the constructor" if exists $opts{constraint};
 	_croak "Class type constraints cannot have a inlining coderef passed to the constructor" if exists $opts{inlined};
 	_croak "Need to supply class name" unless exists $opts{class};
 	
+	if (Type::Tiny::_USE_XS)
+	{
+		my $xsub = Type::Tiny::XS::get_coderef_for("InstanceOf[".$opts{class}."]");
+		$opts{compiled_type_constraint} = $xsub if $xsub;
+	}
+	elsif (Type::Tiny::_USE_MOUSE)
+	{
+		require Mouse::Util::TypeConstraints;
+		my $maker = "Mouse::Util::TypeConstraints"->can("generate_isa_predicate_for");
+		$opts{compiled_type_constraint} = $maker->($opts{class}) if $maker;
+	}
+	
 	return $proto->SUPER::new(%opts);
 }
 
@@ -48,6 +61,13 @@ sub _build_inlined
 {
 	my $self  = shift;
 	my $class = $self->class;
+	
+	if (Type::Tiny::_USE_XS)
+	{
+		my $xsub = Type::Tiny::XS::get_subname_for("InstanceOf[$class]");
+		return sub { my $var = $_[1]; "$xsub\($var\)" } if $xsub;
+	}
+	
 	sub {
 		my $var = $_[1];
 		qq{Scalar::Util::blessed($var) and $var->isa(q[$class])};
@@ -56,11 +76,12 @@ sub _build_inlined
 
 sub _build_default_message
 {
+	no warnings 'uninitialized';
 	my $self = shift;
 	my $c = $self->class;
-	return sub { sprintf 'value "%s" did not pass type constraint (not isa %s)', $_[0], $c } if $self->is_anon;
+	return sub { sprintf '%s did not pass type constraint (not isa %s)', Type::Tiny::_dd($_[0]), $c } if $self->is_anon;
 	my $name = "$self";
-	return sub { sprintf 'value "%s" did not pass type constraint "%s" (not isa %s)', $_[0], $name, $c };
+	return sub { sprintf '%s did not pass type constraint "%s" (not isa %s)', Type::Tiny::_dd($_[0]), $name, $c };
 }
 
 sub _instantiate_moose_type
@@ -121,6 +142,13 @@ sub _build_parent
 	my $self  = shift;
 	my $class = $self->class;
 	
+	# Some classes (I'm looking at you, Math::BigFloat) include a class in
+	# their @ISA to inherit methods, but then override isa() to return false,
+	# so that they don't appear to be a subclass.
+	#
+	# In these cases, we don't want to list the parent class as a parent
+	# type constraint.
+	#
 	my @isa = grep $class->isa($_), do { no strict "refs"; no warnings; @{"$class\::ISA"} };
 	
 	if (@isa == 0)
@@ -140,6 +168,48 @@ sub _build_parent
 	);
 }
 
+*__get_linear_isa_dfs = eval { require mro }
+	? \&mro::get_linear_isa
+	: sub {
+		no strict 'refs';
+		
+		my $classname = shift;
+		my @lin = ($classname);
+		my %stored;
+		
+		foreach my $parent (@{"$classname\::ISA"})
+		{
+			my $plin = __get_linear_isa_dfs($parent);
+			foreach (@$plin) {
+				next if exists $stored{$_};
+				push(@lin, $_);
+				$stored{$_} = 1;
+			}
+		}
+		
+		return \@lin;
+	};
+
+sub validate_explain
+{
+	my $self = shift;
+	my ($value, $varname) = @_;
+	$varname = '$_' unless defined $varname;
+	
+	return undef if $self->check($value);
+	return ["Not a blessed reference"] unless blessed($value);
+	
+	my @isa = @{ __get_linear_isa_dfs(ref $value) };
+	
+	my $display_var = $varname eq q{$_} ? '' : sprintf(' (in %s)', $varname);
+	
+	require Type::Utils;
+	return [
+		sprintf('"%s" requires that the reference isa %s', $self, $self->class),
+		sprintf('The reference%s isa %s', $display_var, Type::Utils::english_list(@isa)),
+	];
+}
+
 1;
 
 __END__
@@ -152,6 +222,11 @@ __END__
 
 Type::Tiny::Class - type constraints based on the "isa" method
 
+=head1 STATUS
+
+This module is covered by the
+L<Type-Tiny stability policy|Type::Tiny::Manual::Policies/"STABILITY">.
+
 =head1 DESCRIPTION
 
 Type constraints of the general form C<< { $_->isa("Some::Class") } >>.
@@ -188,13 +263,13 @@ The class for the constraint.
 
 =item C<constraint>
 
-Unlike Type::Tiny, you should generally I<not> pass a constraint to the
-constructor. Instead rely on the default.
+Unlike Type::Tiny, you I<cannot> pass a constraint coderef to the constructor.
+Instead rely on the default.
 
 =item C<inlined>
 
-Unlike Type::Tiny, you should generally I<not> pass an inlining coderef to
-the constructor. Instead rely on the default.
+Unlike Type::Tiny, you I<cannot> pass an inlining coderef to the constructor.
+Instead rely on the default.
 
 =item C<parent>
 
@@ -273,7 +348,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2013 by Toby Inkster.
+This software is copyright (c) 2013-2014 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
