@@ -14,7 +14,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2013 by Toby Inkster.
+This software is copyright (c) 2013-2014 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
@@ -29,7 +29,7 @@ use warnings;
 use Scalar::Util "looks_like_number";
 use Type::Utils;
 
-use base "Type::Library";
+use Type::Library -base;
 
 declare "String",
 	where { not ref $_ }
